import '../core/rx_impl.dart';

class RxList<T> extends RxListImp<T> {
  RxList([List<T> initial = const []]) : super(initial);

  factory RxList.from(List<T> other) => RxList(List.from(other));

  factory RxList.of(List<T> other) => RxList(List.of(other));

  factory RxList.empty({bool growable = false}) => RxList(List.empty(growable: growable));

  factory RxList.generate(int length, T generator(int index), {bool growable = true}) =>
      RxList(List.generate(length, generator, growable: growable));

  factory RxList.filled(int length, T fill, {bool growable = false}) => RxList((List.filled(length, fill, growable: growable)));

  @override
  void add(T t) {
    value.add(t);
    refresh();
  }

  void addIf(T t, bool test(T t)) {
    if (test(t)) add(t);
  }

  @override
  void addAll(Iterable<T> iterable) {
    value.addAll(iterable);
    refresh();
  }

  void addAllIf(Iterable<T> iterable, bool test()) {
    if (test()) addAll(iterable);
  }

  void addAllWhere(Iterable<T> iterable, bool test(T t)) {
    addAll(iterable.where(test));
  }

  @override
  void insert(int index, T element) {
    value.insert(index, element);
    refresh();
  }

  @override
  void insertAll(int index, Iterable<T> iterable) {
    value.insertAll(index, iterable);
    refresh();
  }

  @override
  bool remove(Object element) {
    bool removed = value.remove(element);
    if (removed) refresh();
    return removed;
  }

  @override
  T removeAt(int index) {
    final t = value.removeAt(index);
    refresh();
    return t;
  }

  @override
  T removeLast() {
    final t = value.removeLast();
    refresh();
    return t;
  }

  @override
  void removeRange(int start, int end) {
    value.removeRange(start, end);
    refresh();
  }

  @override
  void removeWhere(bool Function(T element) test) {
    value.removeWhere(test);
    refresh();
  }

  @override
  void replaceRange(int start, int end, Iterable<T> newContents) {
    value.replaceRange(start, end, newContents);
    refresh();
  }

  @override
  void clear() {
    value.clear();
    refresh();
  }

  @override
  void sort([int Function(T a, T b) compare]) {
    value.sort(compare);
    refresh();
  }

  @override
  void operator []=(int index, T t) {
    value[index] = t;
    refresh();
  }

  @override
  int get length {
    return value.length;
  }

  @override
  T operator [](int index) => value[index];

  @override
  set length(int newLength) {
    value.length = newLength;
    refresh();
  }
}

extension RxListExt<T> on List<T> {
  RxList<T> get obs => RxList(this);
}
