import 'package:yuro_extension/yuro_extension.dart';

import '../core/rx_impl.dart';

class RxMap<K, V> extends RxMapImp<K, V> {
  RxMap([Map<K, V> initial]) : super(initial);

  factory RxMap.from(Map<K, V> other) => RxMap(Map.from(other));

  factory RxMap.of(Map<K, V> other) => RxMap(Map.of(other));

  factory RxMap.identity() => RxMap(Map.identity());

  void addIf(K k, V v, bool test(K k, V v)) {
    if (test(k, v)) {
      value[k] = v;
      refresh();
    }
  }

  @override
  void addAll(Map<K, V> other) {
    value.addAll(other);
    refresh();
  }

  void addAllIf(Map<K, V> other, bool test()) {
    if (test()) addAll(other);
  }

  void addAllWhere(Map<K, V> other, bool test(K k, V v)) {
    addAll(other.where(test));
  }

  @override
  V update(K key, V Function(V value) update, {V ifAbsent()}) {
    final v = value.update(key, update, ifAbsent: ifAbsent);
    refresh();
    return v;
  }

  @override
  void updateAll(V Function(K key, V value) update) {
    value.updateAll(update);
    refresh();
  }

  @override
  V putIfAbsent(K key, V ifAbsent()) {
    if (!value.containsKey(key)) {
      value[key] = ifAbsent();
      refresh();
    }
    return value[key];
  }

  @override
  void addEntries(Iterable<MapEntry<K, V>> newEntries) {
    value.addEntries(newEntries);
    refresh();
  }

  @override
  V remove(Object k) {
    final v = value.remove(k);
    refresh();
    return v;
  }

  @override
  void removeWhere(bool Function(K key, V value) test) {
    value.removeWhere(test);
    refresh();
  }

  @override
  void operator []=(K k, V v) {
    value[k] = v;
    refresh();
  }

  @override
  void clear() {
    value.clear();
    refresh();
  }

  @override
  V operator [](Object k) => value[k];

  @override
  Iterable<K> get keys => value.keys;

  RxMap<K, V> where(bool test(K k, V v)) => RxMap.from(value.where(test));
}

extension RxMapExt<K, V> on Map<K, V> {
  RxMap<K, V> get obs => RxMap<K, V>(this);
}
