import '../core/rx_impl.dart';

abstract class _RxBaseNum<T extends num> extends RxImpl<T> {
  _RxBaseNum(T initial) : super(initial);

  bool operator <(num other) => value < other;

  bool operator <=(num other) => value <= other;

  bool operator >(num other) => value > other;

  bool operator >=(num other) => value >= other;

  bool get isNaN => value.isNaN;

  bool get isNegative => value.isNegative;

  bool get isInfinite => value.isInfinite;

  bool get isFinite => value.isFinite;

  int round() => value.round();

  int floor() => value.floor();

  int ceil() => value.ceil();

  int truncate() => value.truncate();

  double roundToDouble() => value.roundToDouble();

  double floorToDouble() => value.floorToDouble();

  double ceilToDouble() => value.ceilToDouble();

  double truncateToDouble() => value.truncateToDouble();

  int toInt() => value.toInt();

  double toDouble() => value.toDouble();

  String toStringAsFixed(int fractionDigits) => value.toStringAsFixed(fractionDigits);

  String toStringAsExponential([int fractionDigits]) => value.toStringAsExponential(fractionDigits);

  String toStringAsPrecision(int precision) => value.toStringAsPrecision(precision);

  num operator *(num other) => value * other;

  num operator %(num other) => value % other;

  double operator /(num other) => value / other;

  int operator ~/(num other) => value ~/ other;

  num operator -() => -value;

  num remainder(num other) => value.remainder(other);

  num abs() => value.abs();

  num get sign => value.sign;

  num clamp(num lowerLimit, num upperLimit) => value.clamp(lowerLimit, upperLimit);
}

class RxNum extends _RxBaseNum<num> {
  RxNum([num initial]) : super(initial);

  RxNum operator +(num other){
    value += other;
    return this;
  }

  RxNum operator -(num other) {
    value -= other;
    return this;
  }
}

extension RxNumExt on num {
  RxNum get obs => RxNum(this);
}

class RxInt extends _RxBaseNum<int> {
  RxInt([int initial]) : super(initial);

  RxInt operator +(int other){
    value += other;
    return this;
  }

  RxInt operator -(int other) {
    value -= other;
    return this;
  }

  int operator &(int other) => value & other;

  int operator |(int other) => value | other;

  int operator ^(int other) => value ^ other;

  int operator ~() => ~value;

  int operator <<(int shiftAmount) => value << shiftAmount;

  int operator >>(int shiftAmount) => value >> shiftAmount;

  int modPow(int exponent, int modulus) => value.modPow(exponent, modulus);

  int modInverse(int modulus) => value.modInverse(modulus);

  int gcd(int other) => value.gcd(other);

  bool get isEven => value.isEven;

  bool get isOdd => value.isOdd;

  int get bitLength => value.bitLength;

  int toUnsigned(int width) => value.toUnsigned(width);

  int toSigned(int width) => value.toSigned(width);

  String toRadixString(int radix) => value.toRadixString(radix);
}

extension RxIntExt on int {
  RxInt get obs => RxInt(this);
}

class RxDouble extends _RxBaseNum<double> {
  RxDouble(double initial) : super(initial);

  RxDouble operator +(double other){
    value += other;
    return this;
  }

  RxDouble operator -(double other) {
    value -= other;
    return this;
  }
}

extension RxDoubleExt on double {
  RxDouble get obs => RxDouble(this);
}
