import '../core/rx_impl.dart';

class RxSet<T> extends RxSetImpl<T> {
  RxSet([Set<T> initial = const {}]) : super(initial);

  factory RxSet.from(Set<T> other) => RxSet(Set.from(other));

  factory RxSet.of(Set<T> other) => RxSet(Set.of(other));

  factory RxSet.identity() => RxSet(Set.identity());

  @override
  bool add(T element) {
    bool added = value.add(element);
    if (added) refresh();
    return added;
  }

  /// 如果满足[test]条件,则加入
  bool addIf(T element, bool test(T element)) {
    bool added = false;
    if (test(element)) return add(element);
    return added;
  }

  @override
  void addAll(Iterable<T> elements) {
    value.addAll(elements);
    refresh();
  }

  /// 如果满足[test]条件,则全部加入
  void addAllIf(Iterable<T> elements, bool test()) {
    if (test()) addAll(elements);
  }

  /// 将满足[test]条件的elements加入
  void addAllWhere(Iterable<T> elements, bool test(T element)) {
    addAll(elements.where(test));
  }

  @override
  bool remove(Object element) {
    bool removed = value.remove(element);
    if (removed) refresh();
    return removed;
  }

  @override
  void removeWhere(bool test(T element)) {
    value.removeWhere(test);
    refresh();
  }

  @override
  void removeAll(Iterable<Object> elements) {
    value.removeAll(elements);
    refresh();
  }

  @override
  void retainAll(Iterable<Object> elements) {
    value.retainAll(elements);
    refresh();
  }

  @override
  void retainWhere(bool test(T element)) {
    value.retainWhere(test);
    refresh();
  }

  @override
  void clear() {
    value.clear();
    refresh();
  }

  @override
  bool contains(Object element) => value.contains(element);

  @override
  bool containsAll(Iterable<Object> other) => value.containsAll(other);

  @override
  Set<T> intersection(Set<Object> other) => value.intersection(other);

  @override
  Set<T> union(Set<T> other) => value.union(other);

  @override
  Set<T> difference(Set<Object> other) => value.difference(other);

  @override
  Iterator<T> get iterator => value.iterator;

  @override
  int get length => value.length;

  @override
  T lookup(Object element) => value.lookup(element);

  @override
  Set<T> toSet() => value.toSet();
}

extension RxSetExt<T> on Set<T> {
  RxSet<T> get obs => RxSet<T>(this);
}
