import 'dart:async';

import 'package:flutter/material.dart';

import '../core/rx_interface.dart';
import '../core/rx_impl.dart';

class Obx extends _ObxWidget {
  final Widget Function() builder;

  Obx(this.builder, {Key key}) : super(key: key);

  @override
  Widget build() => builder();
}

class ObxValue<T extends RxInterface> extends _ObxWidget {
  final T data;
  final Widget Function(T data) builder;

  ObxValue(this.data, this.builder, {Key key})
      : assert(data != null),
        super(key: key);

  @override
  Widget build() => builder(data);
}

abstract class _ObxWidget extends StatefulWidget {
  const _ObxWidget({Key key}) : super(key: key);

  @override
  _ObxWidgetState createState() => _ObxWidgetState();

  @protected
  Widget build();
}

class _ObxWidgetState extends State<_ObxWidget> {
  RxInterface _observer = RxNotifier();
  StreamSubscription _subscription;

  @override
  void initState() {
    _subscription = _observer.listen(_updateTree);
    super.initState();
  }

  void _updateTree(_) {
    if (mounted) setState(() {});
  }

  @override
  void dispose() {
    _subscription.cancel();
    _observer.close();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final observer = RxInterface.proxy;
    RxInterface.proxy = _observer;
    final result = widget.build();
    if (!_observer.canUpdate) {
      throw '''
      [Obx] build()方法中应该观察一个'Rx'对象, 或者使用对应的'.obs'扩展函数生成的'Rx'对象. 当前支持
      RxInt、RxDouble、RxBool、RxNum、RxList、RxMap、RxSet、Rx<T>.
      ''';
    }

    RxInterface.proxy = observer;
    return result;
  }
}
